// ==============================================================
//                 ORBITER MODULE: euclid
//                  
//
// euclid.cpp
// Control module for euclid vessel class
//
// 
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>
#include <math.h>
#include "OrbiterSoundSDK40.h"

// ==============================================================
// Some vessel parameters
// ==============================================================

const double RCSISP = 2300;
const double RCSTH = 10;
const double ANT_DEPLOY_SPEED = 0.2;
const double ANT_ELBOW_SPEED = 0.05;
const double ANT_WRIST_SPEED = 0.05;
const double FUEL_MASS = 140;
const double EMP_MASS = 2020;

#define YELLOW RGB(255, 255, 80)
#define GREEN RGB(0, 225, 0)

static TOUCHDOWNVTX tdvtx_geardown[3] = {
	{_V( 0,-1, -1.32), 6e4, 2e3, 3.0, 0.2},
	{_V(-0.5, 0.866, -1.32), 6e4, 2e3, 3.0, 0.2},
	{_V( 0.5, 0.866, -1.32), 6e4, 2e3, 3.0, 0.2}
};




// Calculate lift coefficient [Cl] as a function of aoa (angle of attack) over -Pi ... Pi
// Implemented here as a piecewise linear function
//void VLiftCoeff (VESSEL *v, double aoa, double M, double Re, void *context, double *cl, double *cm, double *cd)
//{
//	int i;
//	const int nabsc = 9;
//	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, -2*RAD, 10*RAD,15*RAD,50*RAD,90*RAD,180*RAD};
//	static const double CL[nabsc]  = {0,0,0,0,0,0,0,0,0};
//	static const double CM[nabsc]  = {0,0,0,0,0,0,0,0,0};
//	for (i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
//	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
//	*cl = CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
//	*cm = 0.0;
//	*cd = 0.95 + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
//}

//void HLiftCoeff (VESSEL *v, double aoa, double M, double Re, void *context, double *cl, double *cm, double *cd)
//{
//	int i;
//	const int nabsc = 9;
//	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, -2*RAD, 10*RAD,15*RAD,50*RAD,90*RAD,180*RAD};
//	static const double CL[nabsc]  = {0,0,0,0,0,0,0,0,0};
//	static const double CM[nabsc]  = {0,0,0,0,0,0,0,0,0};
//	for (i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
//	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
//	*cl = CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
//	*cm = 0.0;
//	*cd = 0.95 + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
//}


class euclid: public VESSEL3 {
public:
	
	euclid (OBJHANDLE hVessel, int flightmodel);
	~euclid ();

	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//void clbkPostStep (double simt, double simdt, double mjd);
	void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	int clbkConsumeDirectKey(char *kstate);
	

private:
	void DefineAnimations (void);
	void DoKillRot(void);


	MGROUP_TRANSFORM *MgrpTran[3];
	UINT anim_ant_deploy, anim_ant_elbow, anim_ant_wrist;
	int MyID;
	THRUSTER_HANDLE th_rcs[14], th_group[2];
	PROPELLANT_HANDLE hProp;
	ANIMATIONCOMPONENT_HANDLE Section0, Section1;
	//ATTACHMENTHANDLE hAttach;
	double ant_deploy_proc, ant_elbow_proc, ant_wrist_proc;
	int ant_deploy_status, kilrot_sw;


};

euclid::euclid (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{

}

euclid::~euclid ()
{
int i;
for (i = 0; i < 3; i++)
	{
	delete MgrpTran[i];
	}
}

// Define animation sequences for moving parts
// --------------------------------------------------------------
void euclid::DefineAnimations (void)
{
	// arm
	anim_ant_deploy = CreateAnimation (0);

	static UINT ArrayGrp0[1] = {20};
	MgrpTran[0] = new MGROUP_ROTATE(0, ArrayGrp0, 1, _V(-0.976,0,-1.694), _V(0,1,0), (float)(-110*RAD));
	Section0 = AddAnimationComponent (anim_ant_deploy, 0.0, 1.0, MgrpTran[0]);

	anim_ant_elbow = CreateAnimation (0);
	//gimbals
	static UINT ArrayGrp1[2] = {17,19};
	MgrpTran[1] = new MGROUP_ROTATE(0, ArrayGrp1, 2, _V(-0.976,1.922,-1.694), _V(0,0,1), (float)(-PI2));
	Section1 = AddAnimationComponent (anim_ant_elbow, 0.0, 1.0, MgrpTran[1], Section0);

	anim_ant_wrist = CreateAnimation (0.5);
	// ant
	static UINT ArrayGrp2[3] = {15,16,18};
	MgrpTran[2] = new MGROUP_ROTATE(0, ArrayGrp2, 3, _V(-0.933,1.922,-2.178), _V(1,0,0), (float)(PI));
	AddAnimationComponent (anim_ant_wrist, 0.0, 1.0, MgrpTran[2], Section1);

	

}



// ==============================================================
// euclid class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void euclid::clbkSetClassCaps (FILEHANDLE cfg)
{
	ant_deploy_proc = 1.0;
	ant_deploy_status = 2;
	ant_elbow_proc = 0.0;
	ant_wrist_proc = 0.5;
	kilrot_sw = 0;
	
	DefineAnimations ();

	// physical specs
	SetCW (1, 1, 1, 1);
	SetRotDrag (_V(1,1,1));
	SetPMI (_V(0.6,0.7,0.7));
	SetCrossSections (_V(12,12,9));
	SetSize (3);
	SetEmptyMass (EMP_MASS);
	SetMeshVisibilityMode ((AddMesh ("euclid/euclid")), MESHVIS_EXTERNAL);
	
	SetCameraOffset (_V(0,0,0));
	//SetTouchdownPoints (_V(0.1,0.8,-1.8), _V(0.512,-0.853,-1.8), _V(-0.512,-0.853,-1.8));
	SetTouchdownPoints ( tdvtx_geardown, 3);
	//SetDockParams (_V(0,0,-2), _V(0,0,-1), _V(0,1,0));

	// propellant resources

	hProp = CreatePropellantResource (FUEL_MASS, FUEL_MASS);

	
	// ***************** thruster definitions *******************

	th_rcs[0] = CreateThruster (_V( 0, 0, 2), _V(0,1,0), RCSTH, hProp, RCSISP);
	th_rcs[1] = CreateThruster (_V( 0, 0, 2), _V(0,-1,0), RCSTH, hProp, RCSISP);
	th_rcs[2] = CreateThruster (_V( 0, 0, 2), _V(1,0,0), RCSTH, hProp, RCSISP);
	th_rcs[3] = CreateThruster (_V( 0, 0, 2), _V(-1,0,0), RCSTH, hProp, RCSISP);

	th_rcs[4] = CreateThruster (_V( 0, 0,-2), _V(0,1,0), RCSTH, hProp, RCSISP);
	th_rcs[5] = CreateThruster (_V( 0, 0,-2), _V(0,-1,0), RCSTH, hProp, RCSISP);
	th_rcs[6] = CreateThruster (_V( 0, 0,-2), _V(1,0,0), RCSTH, hProp, RCSISP);
	th_rcs[7] = CreateThruster (_V( 0, 0,-2), _V(-1,0,0), RCSTH, hProp, RCSISP);

	th_rcs[8] = CreateThruster (_V( 0, 2, 0), _V( 1,0,0), RCSTH, hProp, RCSISP);
	th_rcs[9] = CreateThruster (_V( 0, 2, 0), _V(-1,0,0), RCSTH, hProp, RCSISP);
	th_rcs[10] = CreateThruster (_V( 0,-2, 0), _V( 1,0,0), RCSTH, hProp, RCSISP);
	th_rcs[11] = CreateThruster (_V( 0,-2, 0), _V(-1,0,0), RCSTH, hProp, RCSISP);

	th_rcs[12] = CreateThruster (_V(0.978,-1.036,-1.635), _V(0,0,1), RCSTH, hProp, RCSISP);
	th_rcs[13] = CreateThruster (_V(-0.978,1.036,-1.635), _V(0,0,1), RCSTH, hProp, RCSISP);

	

	th_group[0] = th_rcs[ 0];
	th_group[1] = th_rcs[ 5];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[ 1];
	th_group[1] = th_rcs[ 4];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);
 
	th_group[0] = th_rcs[ 3];
	th_group[1] = th_rcs[ 6];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[ 2];
	th_group[1] = th_rcs[ 7];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[ 8];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[ 9];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[12];
	th_group[1] = th_rcs[13];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_FORWARD);

	th_group[0] = th_rcs[12];
	th_group[1] = th_rcs[13];
	CreateThrusterGroup (th_group, 2, THGROUP_MAIN);

	///////////////////////////////////////

	SURFHANDLE tex_rcs = oapiRegisterExhaustTexture ("exhaust_atrcs");
	
	double ex_lng = 1.2;
	double ex_wdt = 0.06;
	AddExhaust (th_rcs[12], ex_lng, ex_wdt, (tex_rcs));
	AddExhaust (th_rcs[13], ex_lng, ex_wdt, (tex_rcs));



	///////////////////////////////////////////////////////
	/////AirFoils

//	CreateAirfoil3 (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 0, 1, 1, 1);
//	CreateAirfoil3 (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 0, 1, 1, 1);

	//////////////////// attach //////////////////////////

//	hAttach = CreateAttachment (true, _V( 0, 0, -2.39), _V( 0,0,-1), _V(0,1,0), "F9");

}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new euclid (hvessel, flightmodel);
}




// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (euclid*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void euclid::clbkPostCreation ()
{
	
	MyID=ConnectToOrbiterSoundDLL(GetHandle());
	ReplaceStockSound(MyID,"Sound\\Vessel\\attsustain.wav",	REPLACE_MAIN_THRUST);
	SoundOptionOnOff(MyID,PLAYGPWS,FALSE);
	SoundOptionOnOff(MyID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	SoundOptionOnOff(MyID,PLAYLANDINGANDGROUNDSOUND,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINAIRCONDITIONING,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINRANDOMAMBIANCE,FALSE);
	SoundOptionOnOff(MyID,DISPLAYTIMER,FALSE);
	SoundOptionOnOff(MyID,PLAYDOCKLANDCLEARANCE,FALSE);
	SoundOptionOnOff(MyID,PLAYRADARBIP,FALSE);
	
}
void euclid::clbkSaveState (FILEHANDLE scn)
{

	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState (scn);

	// custom parameters

	sprintf (cbuf, "%d %0.4f %0.4f %0.4f", ant_deploy_status, ant_deploy_proc, ant_elbow_proc, ant_wrist_proc);
	oapiWriteScenario_string (scn, "ANTN", cbuf);

	sprintf (cbuf, "%d", kilrot_sw);
	oapiWriteScenario_string (scn, "KROT", cbuf);


	
}
void euclid::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "ANTN", 4))
		{
            sscanf (line+4, "%d%lf%lf%lf", &ant_deploy_status, &ant_deploy_proc, &ant_elbow_proc, &ant_wrist_proc);
		}
		else if (!strnicmp (line, "KROT", 4))
		{
            sscanf (line+4, "%d", &kilrot_sw);
		}
		else
		{
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	
	SetAnimation (anim_ant_deploy, ant_deploy_proc);
	SetAnimation (anim_ant_elbow, ant_elbow_proc);
	SetAnimation (anim_ant_wrist, ant_wrist_proc);

	
}






int euclid::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{

//bool cbSelectTarget(void *id, char *str, void *data);

  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) // +shift
	{
	  switch (key) 
	  {
		case OAPI_KEY_NUMPAD5: 
		if (kilrot_sw  == 0)kilrot_sw = 1;
		else kilrot_sw = 0;
		return 1;
	  }
	  return 0;
	} 
  else if (KEYMOD_CONTROL (kstate)) // +ctrl
	{
	return 0;
	}
  else // unmodified keys
	{ 
    switch (key) 
		{
		case OAPI_KEY_G: // ant
		if (ant_deploy_status  == 2)ant_deploy_status = 1;
		return 1;
		}
	return 0;
	}
  
 return 0;
 }


int euclid::clbkConsumeDirectKey(char *kstate)
{
		VECTOR3 avel = _V(0,0,0);
		GetAngularVel(avel);
		double dt = oapiGetSimStep();
		double drot = RAD*2*dt;

if (KEYMOD_SHIFT (kstate))
{
if (KEYDOWN (kstate, OAPI_KEY_NUMPAD2))
		{
		if (avel.x < RAD*10)
			{
			avel.x = avel.x+drot;
			if (avel.x > RAD*10) avel.x = RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD8))
		{
		if (avel.x > -RAD*10)
			{
			avel.x = avel.x-drot;
			if (avel.x < -RAD*10) avel.x = -RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD1))
		{
		if (avel.y < RAD*10)
			{
			avel.y = avel.y+drot;
			if (avel.y > RAD*10) avel.y = RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD3))
		{
		if (avel.y > -RAD*10)
			{
			avel.y = avel.y-drot;
			if (avel.y < -RAD*10) avel.y = -RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD6))
		{
		if (avel.z < RAD*10)
			{			
			avel.z = avel.z+drot;
			if (avel.z > RAD*10) avel.z = RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD4))
		{
		if (avel.z > -RAD*10)
			{
			avel.z = avel.z-drot;
			if (avel.z < -RAD*10) avel.z = -RAD*10;
			}		
		}

		SetAngularVel(avel);
	}
return 0;
}





bool euclid::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{

	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);


	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	//int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	//int w5 = (int)(264*d);
	//int w6 = (int)(280*d);
	//int w7 = (int)(296*d);
	//int w8 = (int)(312*d);
	//int w9 = (int)(328*d);
	//int w10 = (int)(344*d);
	//int w11 = (int)(360*d);
	//int w12 = (int)(376*d);
	//int w13 = (int)(392*d);
	//int w14 = (int)(408*d);
	//int w15 = (int)(424*d);

	{
	skp->SetTextColor(0x0066FF66);
	double m1 = GetEmptyMass();
	double m2 = GetPropellantMass(hProp);
	double mass_fraction = ((m1+m2)/(m1));
	double dv = (RCSISP*(log(mass_fraction)));
	char cbuf[256];
	sprintf(cbuf,"Available dV %0.3f m/s", dv);
	skp->Text ((10+roxl), (wc+royl), cbuf,strlen(cbuf));
	}


	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Shift+NumPad = Reaction Wheel Attitude Control");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	if(kilrot_sw == 1)skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"Shift+NumPad5 = Reaction Wheel KillRot");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}



	if(ant_deploy_status == 2)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"G = Deploy Antenna");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}

	return true;
}


void euclid::clbkPreStep(double SimT, double SimDT, double MJD)
{
	
	if (kilrot_sw  == 1)DoKillRot();

	///////// animations //////////////////////////////////

	// deploy ant
	if (ant_deploy_status == 2) ant_deploy_proc = 1;
	else if (ant_deploy_status == 1) 
	{
		double da = SimDT * ANT_DEPLOY_SPEED;
		ant_deploy_proc = ant_deploy_proc - da;
		if (ant_deploy_proc < 0) 
		{
			ant_deploy_proc = 0;
			ant_deploy_status = 0;
		}
	}
	else if (ant_deploy_status == 0) ant_deploy_proc = 0;
	SetAnimation (anim_ant_deploy, ant_deploy_proc);

	

	if(ant_deploy_status == 0)
	{
	VECTOR3 earth_glob_pos = _V(0,0,0);
	VECTOR3 earth_loc_pos = _V(0,-1,0);
	OBJHANDLE hEarth = oapiGetObjectByName("Earth");
	if (hEarth)
		{
		oapiGetGlobalPos(hEarth, &earth_glob_pos);
		Global2Local(earth_glob_pos, earth_loc_pos);
		}
	normalise(earth_loc_pos);

	double earth_z_angle = atan2(earth_loc_pos.x, earth_loc_pos.y);
	if(earth_z_angle < 0)earth_z_angle = earth_z_angle + PI2;

	double ant_z_angle = ant_elbow_proc * PI2;

	double z_angle_diff = earth_z_angle - ant_z_angle;
	if(z_angle_diff > PI)z_angle_diff = z_angle_diff - PI2;
	else if(z_angle_diff < -PI)z_angle_diff = z_angle_diff + PI2;

	//VECTOR3 earth_loc_zx = earth_loc_pos;
	//earth_loc_zx.y = 0;
	//normalise(earth_loc_zx);

	//VECTOR3 ant_zx = _V(sin(ant_y_angle),0,-cos(ant_y_angle));
	//normalise(ant_zx);

	//VECTOR3 anty_earth_nml = crossp(ant_zx, earth_loc_zx);
	//normalise(anty_earth_nml);
	//sprintf(oapiDebugString(), "x %0.4f y %0.4f z %0.4f",anty_earth_nml.x, anty_earth_nml.y, anty_earth_nml.z);

	int roty_dir = 1;
	//if(anty_earth_nml.y < 0)roty_dir = -1;
	if(z_angle_diff < 0) roty_dir = -1;

	double max_drotz = SimDT * ANT_ELBOW_SPEED * PI2;

	if(max_drotz > abs(z_angle_diff))ant_elbow_proc = earth_z_angle/PI2;
	else
		{
			double da = SimDT * ANT_ELBOW_SPEED * roty_dir;
			ant_elbow_proc = ant_elbow_proc + da;
			
			int aryp = (int) (ant_elbow_proc);
			if(ant_elbow_proc > 1) ant_elbow_proc = ant_elbow_proc - aryp;
			else if(ant_elbow_proc < 0) ant_elbow_proc = ant_elbow_proc - aryp + 1;
		}
		//sprintf(oapiDebugString(), "roty_dir %d y_angle_diff %0.4f earth_y_angle %0.4f ant_y_angle %0.4f",roty_dir, y_angle_diff*DEG, earth_y_angle*DEG, ant_y_angle*DEG);
		SetAnimation (anim_ant_elbow, ant_elbow_proc);

		//////////////////////////// xrot

		double cos_zaxis_earth_angle = dotp(_V(0,0,-1), earth_loc_pos);
		if(cos_zaxis_earth_angle > 1)cos_zaxis_earth_angle = 1;
		else if(cos_zaxis_earth_angle < -1)cos_zaxis_earth_angle = -1;
		double zaxis_earth_angle = acos(cos_zaxis_earth_angle);
		//zaxis_earth_angle = PI05 - zaxis_earth_angle;

		double ant_wrist_angle = ant_wrist_proc * PI;
		double xrot_angle_diff = zaxis_earth_angle - ant_wrist_angle;

		double max_drotx = SimDT * ANT_WRIST_SPEED;
		if(max_drotx > abs(xrot_angle_diff)) ant_wrist_proc = zaxis_earth_angle/PI;
		else
		{
			int rotx_dir = 1;
			if(xrot_angle_diff < 0) rotx_dir = -1;
			double dx = SimDT * ANT_WRIST_SPEED * rotx_dir;

			ant_wrist_proc = ant_wrist_proc + dx;
			if(ant_wrist_proc > 1) ant_wrist_proc = 1;
			else if(ant_wrist_proc < 0) ant_wrist_proc = 0;
		}
		

		SetAnimation (anim_ant_wrist, ant_wrist_proc);

	}

	
}


//void euclid::clbkPostStep (double simt, double simdt, double mjd)
//{
//
//}

void euclid::DoKillRot()
{
		VECTOR3 avel = _V(0,0,0);
		GetAngularVel(avel);
		double dt = oapiGetSimStep();
		double drot = RAD*2*dt;

		if (drot > abs(avel.x)) avel.x = 0;
		else avel.x = avel.x + (drot * ((abs(avel.x))/-avel.x));
		if (drot > abs(avel.y)) avel.y = 0;
		else avel.y = avel.y + (drot * ((abs(avel.y))/-avel.y));
		if (drot > abs(avel.z)) avel.z = 0;
		else avel.z = avel.z + (drot * ((abs(avel.z))/-avel.z));

		SetAngularVel (avel);
		if (avel.x == 0 && avel.y == 0 && avel.z == 0) kilrot_sw = 0;
}



